#!/bin/sh

ZPKG_NAME="GoogleDriveClient"
ZPKG_GDRIVE_CONTROL_SCRIPT="/usr/local/zy-pkgs/bin/gdrive_control.sh"
ZPKG_GDRIVE_CONFIG_DB="/usr/local/zy-pkgs/config/${ZPKG_NAME}/GoogleDrive.db"
ZPKG_GDRIVE_CONFIG_DB_PATH_FOR_FILEYE="/usr/local/zy-pkgs/config/GoogleDrive"
ZPKG_GDRIVE_CONFIG_DB_FOR_FILEYE="${ZPKG_GDRIVE_CONFIG_DB_PATH_FOR_FILEYE}/GoogleDrive.db"
ZPKG_PKGEXE="/usr/local/zy-pkgs/gui/${ZPKG_NAME}/pkgexe"
ZPKG_GDRIVE_TOKEN=".grive"
ZPKG_GDRIVE_STATE=".grive_state"
ZPKG_GDRIVE_SYNC=".grive_syncing"
ZPKG_GDRIVE_USER=".grive_user"
ZPKG_GDRIVE_UPDATE=".grive_update"
ZPKG_GDRIVE_MAX_RUNNING=3
ZPKG_GDRIVE_UPDATE_PERIOD=15

read_json_config()
{
	JSON=`cat "$1" | sed -e 's/[{}]/''/g' | awk -v k="text" '{n=split($0,a,","); for (i=1; i<=n; i++) print a[i]}' | grep "\"$2\":" | sed 's/:/ /1' | awk -F" " '{ print $2 }'`
}

add_sync_path_to_fileye()
{
	ACCOUNT_TABLE_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM sqlite_master WHERE type='table' and name='Account_Info'"`
	if [ $ACCOUNT_TABLE_EXIST -ne 0 ]; then
		ACCOUNT_COUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM Account_Info"`
		if [ $ACCOUNT_COUNT -ne 0 ]; then
			for index in `seq 1 $ACCOUNT_COUNT`; do
				SYNC_PATH=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT sync_path FROM Account_Info ORDER BY rowid ASC LIMIT 1 OFFSET $(($index-1))"`
				$ZPKG_PKGEXE "pkg_gdrive_set_watch_path add \"${SYNC_PATH}\""
			done
		fi
	fi
}

sync_database()
{
cp -af ${ZPKG_GDRIVE_CONFIG_DB} ${ZPKG_GDRIVE_CONFIG_DB_FOR_FILEYE}

# Set all status column to '0' to prevent the old gdrive_update.sh from syncing sync_folder
sqlite3 ${ZPKG_GDRIVE_CONFIG_DB_FOR_FILEYE} "UPDATE Account_Info SET status='0'"
}

# Init GoogleDrive.db
sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
"CREATE TABLE IF NOT EXISTS Account_Info \
(account TEXT PRIMARY KEY NOT NULL, status INTEGER NOT NULL, sync_path TEXT UNIQUE NOT NULL, user TEXT NOT NULL, \
quota_total INTEGER NOT NULL, quota_used INTEGER NOT NULL, quota_trash INTEGER NOT NULL, change_stamp INTEGER NOT NULL)"

# Check Update_Period table exists or not, if not, create it and set default value; otherwise apply value to UPDATE_INTERVAL
UPDATE_TABLE_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM sqlite_master WHERE type='table' and name='Update_Period'"`
if [ $UPDATE_TABLE_EXIST -eq 0 ]; then
	sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "CREATE TABLE IF NOT EXISTS Update_Period (period INTEGER NOT NULL)"
	sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "INSERT INTO Update_Period VALUES ('$ZPKG_GDRIVE_UPDATE_PERIOD')"
else
	ZPKG_GDRIVE_UPDATE_PERIOD=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT period FROM Update_Period LIMIT 1"`
fi

UPDATE_INTERVAL=$(($ZPKG_GDRIVE_UPDATE_PERIOD*60)) 

if [ "$1" = "startup" ]; then
	# fileye reads the old GoogleDrive.db in /etc/zyxel
	sync_database
	
	# Prevent fileye watch folder from being removed due to firmware reset to default
	add_sync_path_to_fileye
fi

while [ 1 ]; do

	sleep $UPDATE_INTERVAL

	ACCOUNT_TABLE_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM sqlite_master WHERE type='table' and name='Account_Info'"`
	if [ $ACCOUNT_TABLE_EXIST -ne 0 ]; then

		ENABLED_COUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM Account_Info WHERE status='1'"`
		if [ $ENABLED_COUNT -eq 0 ]; then
			 continue
		else
			for index in `seq 1 $ENABLED_COUNT`; do

				# Check current running grive, if exceed maximum, check every 3 minutes to see the resource have been released or not
				RUNNING_GRIVE=`ps | grep -c [g]rive`
				while [ $RUNNING_GRIVE -ge $ZPKG_GDRIVE_MAX_RUNNING ]; do
					sleep 180
					RUNNING_GRIVE=`ps | grep -c [g]rive`
				done

				SYNC_PATH=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
						  "SELECT sync_path FROM Account_Info WHERE status='1' ORDER BY rowid ASC LIMIT 1 OFFSET $(($index-1))"`
				GDRIVE_ACCOUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT account FROM Account_Info WHERE sync_path='${SYNC_PATH}'"`
				
				if [ ! -d "${SYNC_PATH}" ] || [ ! -f "${SYNC_PATH}/${ZPKG_GDRIVE_TOKEN}" ]; then
					${ZPKG_GDRIVE_CONTROL_SCRIPT} delete "${GDRIVE_ACCOUNT}" "no" &
					zylogger -s 30 -p 4 -f 30 "${ZPKG_NAME}: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to path \"${SYNC_PATH}\" does not exist"
					continue
				elif [ -f "${SYNC_PATH}/${ZPKG_GDRIVE_SYNC}" ]; then
					continue
				else
					USER=`awk 'NR==1' "${SYNC_PATH}/${ZPKG_GDRIVE_USER}"`
					USER_UID=`awk 'NR==2' "${SYNC_PATH}/${ZPKG_GDRIVE_USER}"`
					MOUNT_POINT=`awk 'NR==3' "${SYNC_PATH}/${ZPKG_GDRIVE_USER}"`

					# Check user
					USER_CHECK=`id "${USER}"`
					if [ $? -ne 0 ]; then
						${ZPKG_GDRIVE_CONTROL_SCRIPT} delete "${GDRIVE_ACCOUNT}" "no" &
						zylogger -s 30 -p 4 -f 30 "${ZPKG_NAME}: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to user \"${USER}\" is deleted"
						continue
					fi

					# Check quota
					QUOTA_CHECK=`repquota -n "${MOUNT_POINT}" | grep "${USER_UID}" | grep +`
					if [ -n "${QUOTA_CHECK}" ]; then
						zylogger -s 30 -p 4 -f 30 "${ZPKG_NAME}: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to exceeding user \"${USER}\" quota"
						continue
					fi

					if [ ! -f "${SYNC_PATH}/${ZPKG_GDRIVE_STATE}" ] || [ -f "${SYNC_PATH}/${ZPKG_GDRIVE_UPDATE}" ]; then
						${ZPKG_GDRIVE_CONTROL_SCRIPT} sync "${SYNC_PATH}" &
					else
						_remote_change_stamp=`${ZPKG_GDRIVE_CONTROL_SCRIPT} dry-run "${SYNC_PATH}"`
						read_json_config "${SYNC_PATH}/${ZPKG_GDRIVE_STATE}" "change_stamp"
						_local_change_stamp=$JSON
						if [ $_remote_change_stamp -gt $_local_change_stamp ]; then
							${ZPKG_GDRIVE_CONTROL_SCRIPT} sync "${SYNC_PATH}" &
						else
							continue
						fi
					fi
				fi
			done
		fi
	else
		continue
	fi
	
done

exit 0
